/*
 * @file Manager_Im.c
 * Module Interface for Module Manager_Im
 */

#include "Manager_Im.h"
#include <stdio.h>

static void Publish_Functional_Service_Availability(Manager_Im__context* context)
{
   ECOA__return_status status;
   Manager_Im_container__available_handle basicMathAvailableHandle;

   status = Manager_Im_container__available__get_write_access(context, &basicMathAvailableHandle);

   if (status == ECOA__return_status_OK || status == ECOA__return_status_DATA_NOT_INITIALIZED)
   {
      *(basicMathAvailableHandle.data) = context->user.basicMathServiceAvailable;
   }

   ECOA__log log;

   if (context->user.basicMathServiceAvailable)
   {
      log.current_size = sprintf((char *) &log.data, "- - - - Publishing basicMath service available");
   }
   else
   {

      log.current_size = sprintf((char *) &log.data, "- - - - Publishing basicMath service unavailable");
   }
   Manager_Im_container__log_info(context, log);

   status = Manager_Im_container__available__publish_write_access(context, &basicMathAvailableHandle);
}

void Manager_Im__INITIALIZE__received(Manager_Im__context *context)
{
   context->user.basicMathState = UNINITIALIZED;
   context->user.complexMathState = UNINITIALIZED;

   context->user.basicMathServiceAvailable = ECOA__FALSE;
   Publish_Functional_Service_Availability(context);
}

void Manager_Im__START__received(Manager_Im__context *context)
{
   /* User Code Here */
}

void Manager_Im__STOP__received(Manager_Im__context *context)
{
   /* User Code Here */
}

void Manager_Im__SHUTDOWN__received(Manager_Im__context *context)
{
   /* User Code Here */
}

void Manager_Im__publish__received(Manager_Im__context *context)
{
   ECOA__log log;

   context->user.count++;

   ECOA__uint32 ID;

   // Initialize modules on count 1
   if (context->user.count == 1)
   {
      log.current_size = sprintf((char *) &log.data, "1 - - - - requesting initialisation of modules");
      Manager_Im_container__log_info(context, log);

      Manager_Im_container__initializeBasicMath__request_async(context, &ID);
      Manager_Im_container__initializeComplexMath__request_async(context, &ID);
   }
   // Start modules on count 2
   if (context->user.count == 2)
   {
      log.current_size = sprintf((char *) &log.data, "2 - - - - requesting start of modules");
      Manager_Im_container__log_info(context, log);

      Manager_Im_container__startBasicMath__request_async(context, &ID);
      Manager_Im_container__startComplexMath__request_async(context, &ID);
   }
   // Reinitialize modules on count 6 (set to not running...)
   if (context->user.count == 6)
   {
      log.current_size = sprintf((char *) &log.data, "4 - - - - requesting reinitialise of modules");
      Manager_Im_container__log_info(context, log);
      context->user.basicMathState = UNINITIALIZED;
      context->user.complexMathState = UNINITIALIZED;
      Manager_Im_container__initializeBasicMath__request_async(context, &ID);
      Manager_Im_container__initializeComplexMath__request_async(context, &ID);
      // Publish functional availability of service.
      context->user.basicMathServiceAvailable = ECOA__FALSE;
      context->user.count = 0;
   }

   // Always republish the current availability
   Publish_Functional_Service_Availability(context);
}

void Manager_Im__initializeBasicMath__response_received
   (Manager_Im__context* context,
    const ECOA__uint32 ID, const ECOA__return_status status)
{
   context->user.basicMathState = INITIALIZED;
}
void Manager_Im__initializeComplexMath__response_received
   (Manager_Im__context* context,
    const ECOA__uint32 ID, const ECOA__return_status status)
{
   context->user.complexMathState = INITIALIZED;
}
void Manager_Im__startBasicMath__response_received
   (Manager_Im__context* context,
    const ECOA__uint32 ID, const ECOA__return_status status)
{
   context->user.basicMathState = RUNNING;
   context->user.basicMathServiceAvailable = ECOA__TRUE;
}
void Manager_Im__startComplexMath__response_received
   (Manager_Im__context* context,
    const ECOA__uint32 ID, const ECOA__return_status status)
{
   context->user.complexMathState = RUNNING;
}
